@@ -8,6 +8,8 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <lib/prototypes.h>
+
+#include "alloc.h"
 #include "run_part.h"
 #include "shadowlog_internal.h"
 
@@ -57,7 +59,7 @@ int run_parts (const char *directory, const char *name, const char *action)
 		struct stat sb;
 
 		path_length=strlen(directory) + strlen(namelist[n]->d_name) + 2;
-		char *s = (char*)malloc(path_length);
+		char *s = MALLOC(path_length, char);
 		if (!s) {
 			printf ("could not allocate memory\n");
 			for (; n<scanlist; n++) {
