@@ -182,6 +182,6 @@ struct spwd *sgetspent (const char *string)
 	return (&spwd);
 }
 #else
-extern int errno;		/* warning: ANSI C forbids an empty source file */
+extern int ISO_C_forbids_an_empty_translation_unit;
 #endif
 
